from AbstractSongList import AbstractSongList
from enviroment import env
import gtk
import glib
    
class PlayList (AbstractSongList):
    """
    Represent the PlayList
    """
    
    ICON_SIZE = 80
    
    def __init__(self, view):
        AbstractSongList.__init__(self, view)
        self.playing_song = None
    
    def create_model(self):
        # Model: Song, Info, Image, Tooltip, ...
        return gtk.ListStore(object, str, gtk.gdk.Pixbuf, str, str, int)
    
    def get_sliced_string(self, str, max):
        sliced = str[0:max - 3]
        if len(sliced) < len(str):
            return sliced + "..."
        else:
            return str
        
    def create_view(self):
        self.get_view().set_selection_mode(gtk.SELECTION_MULTIPLE)
        self.get_view().set_reorderable(True)
        
        self.get_view().set_markup_column(1)
        self.get_view().set_pixbuf_column(2)
        self.get_view().set_tooltip_column(3)
        self.get_view().set_item_width(90);
        self.get_view().set_columns(99999)
        
        self.get_view().add_events(gtk.gdk.SCROLL_MASK)
        self.get_view().connect("scroll-event", self.on_mouse_wheel_scroll)
        
    def on_mouse_wheel_scroll(self, widget, event):
        # We need to assume that the IconView has a ScrolledWindow as parent 
        adjustment = self.get_view().get_parent().get_hadjustment()
        if event.direction == gtk.gdk.SCROLL_UP:
            newvalue = adjustment.get_value() - adjustment.get_step_increment()
            if newvalue < 0:
                newvalue = 0
        else:
            newvalue = adjustment.get_value() + adjustment.get_step_increment()
            if newvalue > adjustment.get_upper() - adjustment.get_page_size():
                newvalue = adjustment.get_upper() - adjustment.get_page_size()
        adjustment.set_value(newvalue)
        
    def get_selected_rows(self):
        return self.get_view().get_selected_items()
    
    def get_playing_song(self):
        """
        Returns the current playing song or None if not playing nothing.
        """
        return self.playing_song
    
    def set_playing_song(self, song):
        """
        Set the current playing song.
        @param song: The song or None to indicate that there is no song playing
        """
        if self.playing_song != None:
            oldpath = self.get_song_path(self.playing_song)
            if self.playing_song.get_cover_pixbuf() != None:
                self.get_model()[oldpath][2] = self.create_cornered_image(self.playing_song.get_cover_pixbuf())
        
        if song != None:
            path = self.get_song_path(song)
            if song.get_cover_pixbuf() != None:
                self.get_model()[path][2] = self.create_composited_image(self.create_cornered_image(song.get_cover_pixbuf()))
            self.get_view().scroll_to_path(path, False, 0, 0) 
        
        
        self.playing_song = song
    
    def append_song(self, song):
        # Change song local ID to fix the search by song
        song = song.clone()
        
        song_string = "<span font_size='small'>{title}</span>\n<span font_size='small' fgcolor='#555555'>{artist}</span>".format(
            title = self.get_sliced_string(glib.markup_escape_text(song.get_title()), 25),
            artist = self.get_sliced_string(glib.markup_escape_text(song.get_artist()), 20)
        )
        
        tooltip = _("<b>Title:</b> {title}\n<b>Artist:</b> {artist}\n<b>Album:</b> {album}\n<b>Year:</b> {year}").format(
            title = glib.markup_escape_text(song.get_title()),
            artist = glib.markup_escape_text(song.get_artist()),
            album = glib.markup_escape_text(song.get_album()),
            year = song.get_year()
        )
        
        initial_pixbuf = None
        must_download = None
        if song.get_cover_pixbuf() == None:
            initial_pixbuf = self.create_loading_track_icon()
            must_download = True
        else:
            initial_pixbuf = self.create_cornered_image(song.get_cover_pixbuf())
            must_download = False
        
        self.get_model().append([
            song,
            song_string,
            initial_pixbuf,
            tooltip,
            song.get_id(),
            400
        ])
        
        if must_download == True:
            song.connect("cover-downloaded", self.on_song_cover_downloaded)
            song.download_cover()
            
    def on_song_cover_downloaded(self, song):
        row = self.get_song_row(song)
        if row != None:
            image = self.create_cornered_image(song.get_cover_pixbuf())
            if song.equals(self.get_playing_song()) == True:
                row[2] = self.create_composited_image(image)
            else:
                row[2] = image
        
    def create_loading_track_icon(self):
        default = gtk.gdk.pixbuf_new_from_file("%s/data/loading.png" % env().BASEPATH)
        return self.scale_pixbuf(default)

    def create_cornered_image(self, pixbuf):
        pixbuf = pixbuf.copy()
        pixbuf = self.scale_pixbuf(pixbuf)
        corners = self.scale_pixbuf(gtk.gdk.pixbuf_new_from_file("%s/data/corners.png" % env().BASEPATH))
        corners.composite(pixbuf, 0, 0, pixbuf.props.width, pixbuf.props.height,
                          0, 0, 1.0, 1.0, gtk.gdk.INTERP_HYPER, 255)
        return pixbuf
    
    def create_composited_image(self, pixbuf):
        """
        Composite play.png with the pixbuf
        """
        pixbuf = pixbuf.copy()
        play = self.scale_pixbuf(gtk.gdk.pixbuf_new_from_file('%s/data/play.png' % env().BASEPATH))
        play.composite(pixbuf, 0, 0, pixbuf.props.width, pixbuf.props.height,
                          0, 0, 1.0, 1.0, gtk.gdk.INTERP_HYPER, 255)
        return pixbuf
    
    def scale_pixbuf(self, pixbuf):
        return pixbuf.scale_simple(PlayList.ICON_SIZE, PlayList.ICON_SIZE, gtk.gdk.INTERP_HYPER)
